/*******************************************************************************
 buckets_t.h -

 Copyright (C) 2010-2022 CodeShop B.V.
 http://www.code-shop.com

 For licensing see the LICENSE file
******************************************************************************/

#ifndef BUCKETS_T_H_INCLUDED
#define BUCKETS_T_H_INCLUDED

#include "mod_streaming_export.h"
#include "mp4_error.h"
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct bucket_t bucket_t;

struct headers_t
{
  char const* content_type_;

  // when "cache_control" is set, it is copied to the "Cache-Control" header
  // field. Otherwise the "Cache-Control" may be based on the "expires_at".
  char const* cache_control_;

  // ETag is fixed size string "usp-1234ABCD", where last 8 chars are hex
  // uint32_t hash value of all heap buckets.
  char etag_[32];

  // when set, the "updated_at" is copied to the "Last-Modified" header field.
  uint64_t updated_at_;

  // when set, the "updated_at_experimental" is copied to the
  // "X-USP-Last-Modified" header field.
  uint64_t updated_at_experimental_;

  // Creates a "Cache-Control: max-age=%" and "Expires" header field by using
  // the "expires_at". See: https://tools.ietf.org/html/rfc7234
  uint64_t expires_at_;

  // Creates a "Sunset" header indicating when fragment will be purged from
  // archive. See: https://tools.ietf.org/html/rfc8594
  uint64_t sunset_at_;

  // when "location" is set, it is copied to the "Location" header field and
  // a redirect is issued.
  char* location_;

  //
  char x_usp_header1[256];
  char x_usp_header2[256];
  char link_[512];
};

struct buckets_t
{
  // the bucket list that is output
  bucket_t* bucket_;
};

typedef struct buckets_t buckets_t;

MP4_DLL_EXPORT extern char const* cache_control_no_cache;

MP4_DLL_EXPORT void bucket_insert_head(buckets_t* buckets, bucket_t* bucket);
MP4_DLL_EXPORT void bucket_insert_tail(buckets_t* buckets, bucket_t* bucket);

// returns the size of the buckets by accumulating the size of all the buckets.
// If a bucket with unknown length (UINT64_MAX) is encountered, then UINT64_MAX
// is returned.
MP4_NODISCARD MP4_DLL_EXPORT uint64_t buckets_size(buckets_t const* buckets);

// returns the size of the buckets by accumulating the size of all the buckets.
// If a bucket with unknown length (UINT64_MAX) is encountered, a read-ahead
// replaces it by one or more buckets of known size.
MP4_NODISCARD MP4_DLL_EXPORT fmp4_result
buckets_establish_size(buckets_t const* buckets, uint64_t* out_size);

MP4_DLL_EXPORT void buckets_clear(buckets_t* buckets);

MP4_DLL_EXPORT char* file_url_to_path(char* filename);

#ifdef __cplusplus
}
#endif

#endif
